{
   createchars.pp, creates character images from a single image
   Copyright (c) 2011. Dejan Boras

   Started On:    12.03.2011.
   Last Update:   12.03.2011.

   NOTE: Origin of the images is set to top-left, since imgCopyArea does not yet
   handle images with other types of origins.
}

{$MODE OBJFPC}{$H+}
PROGRAM createchars;

   USES StringUtils, uError, uFile,
      uImage, imguTGA, imguwTGA;

CONST
   CHAR_WIDTH = 16;
   CHAR_HEIGHT = 16;

   HCHARS   = 16;
   VCHARS   = 8;

CONST
   imgfn = 'bigchars.tga';

VAR
   dstfn: string; {name of the destination image}

   img: imgTImage; {source image}
   tmpimg: imgTImage; {temporary image}

   {row, column, current image}
   i, j, n: longint;

BEGIN
   {load the source image}
   imgLoad(img, imgfn);
   if(imgError = 0) then begin
      writeln('Loaded image: ', imgfn);
      writeln('Pixelformat: ', imgPIXFName(img.PixF));
      writeln('Dimensions: ', img.Width, 'x', img.Height);

      {transform image if it is not BGRA}
      if(img.PixF <> PIXF_BGRA) then begin
         imgTransform(img, PIXF_BGRA);
         if(imgError <> 0) or (img.PixF <> PIXF_BGRA) then begin
            writeln('Error: Could not transform image to BGRA pixel format.'); halt(1);
         end;
      end;

      {create a new temporary image}
      imgCreateBlank(tmpimg, CHAR_WIDTH, CHAR_HEIGHT, PIXF_BGRA);
      if(imgError <> 0) then begin
         writeln('Error(',imgError,'): Failed to create new temporary image.'); halt(1);
      end;

      writeln('Creating characters...');

      imgSetOrigin(img, imgcORIGIN_TL);
      imgSetOrigin(tmpimg, imgcORIGIN_TL);

      {go through all characters}
      for i := 0 to (VCHARS-1) do
         for j := 0 to (HCHARS-1) do begin
            n := (i*HCHARS)+j;

            {create a new image by copying the required part from source}
            imgSetOrigin(tmpimg, imgcORIGIN_TL);
            imgCopyArea(img, tmpimg, {sxy}j*CHAR_WIDTH, i*CHAR_HEIGHT, {dxy}0, 0, {dim}CHAR_WIDTH, CHAR_HEIGHT);
            if(imgError <> 0) then begin
               writeln('Error(',imgError,') copying image nr. ', n); halt(1);
            end;
            imgSetOrigin(tmpimg, imgcORIGIN_BL);

            {write the new image}
            imgWrite(tmpimg, dStr(n)+'.tga');
            if(imgError <> 0) then begin
               writeln('Error(',imgError,',',fError,',',ioE,'): Creating image ', dstfn); halt(1);
            end;
         end;

      writeln('Done.');
   end else
      writeln('Error(',imgError,',',fError,',',ioE,'): Failed to load ', imgfn);
END.

