{
   frmuFav, favorites form
   Copyright (c) 2011. Dejan Boras

   This file is part of OpenArena Name Changer (oanc).

   OpenArena Name Changer (oanc) is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   OpenArena Name Changer (oanc) is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with OpenArena Name Changer (oanc). If not, see <http://www.gnu.org/licenses/>.

   Started On:    05.03.2011.
   Last Update:   29.04.2011.
}

{$MODE OBJFPC}{$H+}
UNIT frmuFav;

INTERFACE

USES
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Buttons, Grids,
  {uoanc}
  uoanc, frmuPreview;

TYPE

  { TfrmFav }

  TfrmFav = class(TForm)
     btnSelect: TBitBtn;
     btnRemove: TBitBtn;
     btnCancel: TBitBtn;
     shpPrv: TShape;
     sgNames: TStringGrid;
     procedure btnCancelClick(Sender: TObject);
     procedure btnRemoveClick(Sender: TObject);
     procedure btnSelectClick(Sender: TObject);
     procedure FormCreate(Sender: TObject);
     procedure FormResize(Sender: TObject);
     procedure sgNamesResize(Sender: TObject);
     procedure sgNamesSelection(Sender: TObject; aCol, aRow: Integer);
     procedure shpPrvMouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure shpPrvPaint(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

VAR
  frmFav: TfrmFav;

{add a string to favorites}
procedure AddToFav(const name: string);
{find the specified favorite}
function FindFav(const name: string): longint;

IMPLEMENTATION

CONST
   SECTION_FAV = 'Favorites';
   SECTION_FAV_COUNT = 'Count';

{$R *.lfm}

function GetSelectedName(): string;
var
   s: TStrings;
   sel: longint;

begin
   result := '';

   sel := frmFav.sgNames.Row;
   if(sel > 0) then begin
      s := frmFav.sgNames.Rows[sel];
      result := s[0];
   end;
end;

procedure Preview();
begin
   PreviewName(GetSelectedName(), frmFav.shpPrv);
end;

{ TfrmFav }

procedure TfrmFav.btnCancelClick(Sender: TObject);
begin
   Close();
end;

procedure TfrmFav.btnRemoveClick(Sender: TObject);
var
   sel: longint;
   sg: TStringGrid;
   source: longint;

begin
   sg := frmFav.sgNames;
   sel := sg.Row;
   if(sel > 0) then begin
      source := sg.RowCount-1;

      if(sel <> source) then
         sg.Rows[sel].Assign(sg.Rows[source]);

      sg.RowCount := sg.Rowcount-1;

      Preview();
   end;
end;

procedure TfrmFav.btnSelectClick(Sender: TObject);
var
   sel: longint;

begin
   sel := frmFav.sgNames.Row;

   if(sel > 0) then begin
      playername := frmFav.sgNames.Rows[sel].Strings[0];
      Close();
   end else
      MessageDlg('Error', 'No name is selected.', mtWarning, [mbOK], 0);
end;

procedure TfrmFav.FormCreate(Sender: TObject);
begin
   sgNames.ColWidths[0] := sgNames.Width-4;
end;

procedure TfrmFav.FormResize(Sender: TObject);
begin
   sgNames.Width := frmFav.ClientWidth - (sgNames.Left*2);
   shpPrv.Width := frmFav.ClientWidth - (shpPrv.Left*2);

   btnCancel.Top := frmFav.ClientHeight - (btnCancel.Height+8);
   btnRemove.Top := frmFav.ClientHeight - (btnRemove.Height+8);
   btnSelect.Top := frmFav.ClientHeight - (btnSelect.Height+8);
   shpPrv.Top := btnSelect.Top - (shpPrv.Height + 4);

   sgNames.Height := shpPrv.Top - sgNames.Top - 4;
end;

procedure TfrmFav.sgNamesResize(Sender: TObject);
begin
   sgNames.Columns[0].Width := sgNames.ClientWidth;
end;

procedure TfrmFav.sgNamesSelection(Sender: TObject; aCol, aRow: Integer);
begin
   frmFav.shpPrv.Invalidate();
   Preview();
end;

procedure TfrmFav.shpPrvMouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   if(Button = mbLeft) then begin
      sPreviewName := GetSelectedName();
      frmPreview.ShowModal();
   end;
end;

procedure TfrmFav.shpPrvPaint(Sender: TObject);
begin
   Preview();
end;

{ MANAGEMENT }

procedure AddToFav(const name: string);
var
  sl: TStringList;
  sg: TStringGrid;
  rc: longint;

begin
   sg := frmFav.sgNames;

   rc := sg.RowCount+1;
   sg.RowCount := rc;

   sl := TStringList.Create();
   sl.Add(name);

   sg.Rows[rc-1].Assign(sl);

   sl.Free();
end;

function FindFav(const name: string): longint;
var
   i: longint;
   sg: TStringGrid;

begin
   sg := frmFav.sgNames;
   result := -1;

   if(sg.RowCount > 0) then
   for i := 0 to (sg.RowCount-1) do
      if(sg.Rows[i].Strings[0] = name) then exit(i);
end;

{ CONFIGURATION }
procedure LoadCfg();
var
   i, n: longint;
   ns, s: string;

begin
   n := iniCfg.ReadInteger(SECTION_FAV, SECTION_FAV_COUNT, 0);
   if(n > 0) then begin
      for i := 1 to n do begin
         str(i, ns);
         s := urlDecode(iniCfg.ReadString(SECTION_FAV, ns, ''));

         if(length(s) > 0) then begin
            AddToFav(s);
         end;
      end;
   end;
end;

procedure SaveCfg();
var
   sg: TStringGrid;
   n, i: longint;
   ns: string;

begin
   sg := frmFav.sgNames;
   n := sg.RowCount-1;

   iniCfg.EraseSection('Favorites');
   iniCfg.WriteInteger(SECTION_FAV, SECTION_FAV_COUNT, n);

   if(n > 0) then begin
      for i := 1 to (n) do begin
         str(i, ns);
         iniCfg.WriteString(SECTION_FAV, ns, urlEncode(frmFav.sgNames.Rows[i].Strings[0]));
      end;
   end;
end;

INITIALIZATION
   LoadFavorites := @LoadCfg;
   SaveFavorites := @SaveCfg;
END.

